@@ -1,6 +1,6 @@
 ;;; org-mouse.el --- Better mouse support for org-mode
 
-;; Copyright (C) 2006-2012 Free Software Foundation, Inc.
+;; Copyright (C) 2006-2013 Free Software Foundation, Inc.
 
 ;; Author: Piotr Zielinski <piotr dot zielinski at gmail dot com>
 ;; Maintainer: Carsten Dominik <carsten at orgmode dot org>
@@ -656,11 +656,11 @@ This means, between the beginning of line and the point."
 	 ["All Clear" (org-mouse-for-each-item
 		       (lambda ()
 			 (when (save-excursion (org-at-item-checkbox-p))
-			   (replace-match "[ ]"))))]
+			   (replace-match "[ ] "))))]
 	 ["All Set" (org-mouse-for-each-item
 		     (lambda ()
 		       (when (save-excursion (org-at-item-checkbox-p))
-			 (replace-match "[X]"))))]
+			 (replace-match "[X] "))))]
 	 ["All Toggle" (org-mouse-for-each-item 'org-toggle-checkbox) t]
 	 ["All Remove" (org-mouse-for-each-item
 			(lambda ()
@@ -1056,7 +1056,7 @@ This means, between the beginning of line and the point."
 	  ["Convert" org-agenda-convert-date
 	   (org-agenda-check-type nil 'agenda 'timeline)]
 	  "--"
-	  ["Create iCalendar file" org-export-icalendar-combine-agenda-files t])
+	  ["Create iCalendar file" org-icalendar-combine-agenda-files t])
 	 "--"
 	 ["Day View" org-agenda-day-view
 	  :active (org-agenda-check-type nil 'agenda)
